/* Spaceship controls.
	Like Tank Movement, you can turn and accelerate forward.
	However, there is no friction in space, so your velocity doesn't turn with you.
	And you can't accelerate backwards.
*/

#include "../AtomForward.dm"
#include "../Velocity.dm"
#include "PlayerLoop.dm"
#include "PlayerMoveInput.dm"

// Make the world bigger and see more.
world
	view = 15

mob/player
	icon_state = "arrow" // Arrows show direction best.

	transform = matrix(24/32, 0, 0, 0, 24/32, 0)
	bound_width = 16
	bound_height = 16
	bound_x = 8
	bound_y = 8

	var
		tmp
			turning = 0
			accelerating = FALSE

	proc
		// How fast you turn, in degrees per decisecond.
		TurnRate()
			return 15

		// How fast you accelerate, in pixels per decisecond... per decisecond. (p/ds^2)
		ThrustForce()
			return 3

	UpdateMoveInput()
		..()
		turning = move_input.x // Turn with horizontal move input.
		accelerating = move_input.y > 0 // Accelerate with positive vertical move input.

	PlayerUpdate()
		..()
		// Turn the forward direction and keep it normalized.
		if(turning)
			Turn(turning * TurnRate() * PlayerLoop.tick_lag)

		// Accelerate when accelerating.
		if(accelerating)
			Accelerate(Forward() * ThrustForce())

		// If velocity is too small to move, and the ship isn't trying to go faster, just zero it completely.
		// Makes it easier to come to a full stop.
		if(!accelerating && velocity && velocity.SquareMagnitude() < 1)
			SetVelocity(Vector2.Zero)
