/* Makes the camera zoom in when moving slowly and zoom out when moving fast.
	Speed is based on velocity by default but can be changed in ZoomCameraSpeed.
	Try this with TankControls or SpaceshipMovement.
*/

#include "PlaneTransform.dm"
#include "Velocity.dm"

mob/player
	var
		tmp
			// Zoom level when still (not moving).
			still_zoom = 1.75

			// Zoom level when moving at the speed limit.
			fast_zoom = 1.5

			// Speed limit.
			zoom_speed_limit = 24

			matrix/zoom_matrix

	Login()
		..()
		SetVelocity(Velocity())

	SetVelocity()
		..()
		var matrix/new_zoom_matrix = matrix() * clamp(remap(ZoomCameraSpeed(), 0, zoom_speed_limit, still_zoom, fast_zoom), fast_zoom, still_zoom)
		if(client?.world_plane_master)
			animate(client.world_plane_master,
				time = world.tick_lag,
				flags = ANIMATION_END_NOW | ANIMATION_PARALLEL | ANIMATION_RELATIVE,
				transform = new_zoom_matrix / zoom_matrix
			)
		zoom_matrix = new_zoom_matrix

	proc
		/* How fast is the camera moving?
		*/
		ZoomCameraSpeed()
			return Velocity().Magnitude()

		clamp(value, low, high)
			return min(max(value, low), high)

		remap(value, low1, high1, low2, high2)
			return low2 + (value - low1) * (high2 - low2) / (high1 - low1)
