#include "Bullet.dm"

obj/gun
	icon_state = "rect"
	color = "black"
	transform = matrix(8/32, 0, 0, 0, 1, 32)
	layer = MOB_LAYER - 0.1
	appearance_flags = RESET_TRANSFORM

	var
		// Distance from the center of the shooter to the tip of the muzzle, in pixels.
		muzzle_distance = 48

		// Initial bullet speed, in pixels per tick.
		// Try some really large values to see how built-in collision doesn't
		//  let fast movers tunnel through obstacles.
		bullet_speed = 64

		// Shots per second.
		fire_rate = 10

		// Degrees of random spread for bullets.
		spread = 5

		// Fire modes.
		const
			FullAuto = 1 // Pew pew pew pew pew pew pew...
			SemiAuto = 2 // Pew
			Burst = 3 // Pew pew pew

		// One of the above constants. Try them all.
		fire_mode = FullAuto

		// How many pews per trigger pull when in Burst mode.
		shots_per_burst = 3

		tmp
			trigger_pulled = FALSE // Trigger repeat prevention
			firing = FALSE // Loop stack prevention

	proc
		PullTrigger(mob/shooter)
			if(!trigger_pulled)
				trigger_pulled = TRUE
				PulledTrigger(shooter)

		ReleaseTrigger(mob/shooter)
			if(trigger_pulled)
				trigger_pulled = FALSE
				ReleasedTrigger(shooter)

		PulledTrigger(mob/shooter)
			set waitfor = FALSE

			// If bursting, reset the burst.
			if(fire_mode == Burst)
				shots_per_burst = initial(shots_per_burst)

			if(!firing)
				firing = TRUE

				switch(fire_mode)
					if(FullAuto)
						// Shoot until the trigger is released.
						while(trigger_pulled)
							Shoot(shooter)
							sleep 10/fire_rate

					if(SemiAuto)
						// Shoot once and no faster than the fire rate.
						while(trigger_pulled)
							Shoot(shooter)
							ReleaseTrigger(shooter)
							sleep 10/fire_rate

					if(Burst)
						// Shoot a burst regardless of the trigger state.
						while(shots_per_burst > 0)
							Shoot(shooter)
							shots_per_burst--
							sleep 10/fire_rate

				firing = FALSE

		ReleasedTrigger(mob/shooter)

		Spread()
			return spread * pick(-1, 1) * rand()**2

		MuzzleVelocity()
			return Forward().Turn(Spread()).ToMagnitude(bullet_speed)

		ToMuzzle()
			return Forward() * muzzle_distance

		// Fire a single shot.
		Shoot(mob/shooter)
			// Spawn a bullet at the shooter's position,
			// give it an initial velocity with some random spread,
			// and move it to the muzzle of the gun.
			new/obj/bullet(shooter, shooter.z, shooter.Velocity() + MuzzleVelocity()).PixelMove(ToMuzzle())

			// Moving it to the muzzle from the shooter's center is better than
			// spawning directly at the muzzle since the muzzle may be sticking through
			// some obstacles.
